﻿Imports System.Windows.Forms
Imports Inventor

'Namen der benötigten iProperties
'Public Enum Params
'    Title
'    Project
'    Company
'End Enum

Public Class ClassMain
    Friend Shared ProjList As New List(Of Projekt) 'Liste mit gespeicherten Projekten (bis 8 Stück)
    Public Shared WahlProjekt As Projekt 'Das ausgewählte Projekt mit den iProps

#Region "Inventor Instanzen"
    Shared IV_App As Inventor.Application
    Shared IV_Doc As Inventor.Document 'IV-Dokument allgemein
    Shared IV_Part As Inventor.PartDocument 'Bauteil-Dokument
    Shared IV_BG As Inventor.AssemblyDocument 'Baugruppendokument
    Shared PropSets As Inventor.PropertySets 'iProperties des Dokumentes
#End Region

    Shared iPropNotwendig() As String = {"Title", "Project"}
    Shared iPropSonst() As String = {"Company"}
    Shared NamenListe As New List(Of String)

    ''' <summary>
    ''' Diese Sub steuert den gesamten Ablauf des AutoFill
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Sub Start()
        'Inventor Application laden
        Try
            IV_App = CType(System.Runtime.InteropServices.Marshal.GetActiveObject("Inventor.Application"), Inventor.Application) 'Gibt die aktive IV-App zurück
            IV_Doc = IV_App.ActiveDocument 'gibt das aktuelle Dokument der IV_App zurück
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Inventor ist nicht gestartet")
            Exit Sub
        End Try

        'Test auf noch auszufüllende iProperties
        If Not iPropDoc_leer(iPropNotwendig) Then
            Exit Sub 'Die geforderten iProps sind ausgefüllt
        End If
        'Alle iProp-Namen in Liste zusammenfassen
        NamenListe.AddRange(iPropNotwendig)
        NamenListe.AddRange(iPropSonst)

        'gespeicherte Projektsätze aus den Settings laden
        Projektliste_Load()
        If ProjList.Count < 1 Then
            ProjList = New List(Of Projekt)
            ProjektNeu()
        End If

        'Auswahldialog öffnen
        If ProjList.Count > 0 Then
            Dim Auswahl As New Dialg_ProjAusw
            If Auswahl.ShowDialog = DialogResult.OK Then
                'Auswahl übernehmen
                WahlProjekt = ProjList.Item(Auswahl.index)
                'ausgewähltes Projekt aus der Liste entfernen
                ProjList.Remove(WahlProjekt)
                'und an 1. Stelle wieder einfügen
                ProjList.Insert(0, WahlProjekt)

                'Liste in My.Settings speichern
                Projektliste_Save()
                'Propertties in oDoc schreiben
                iProp_schreiben(NamenListe)
            Else
                WahlProjekt = Nothing
            End If
        End If
    End Sub

    ''' <summary>
    ''' Schreibt die Properties des Projektes in das Dokument
    ''' </summary>
    ''' <param name="Namen"></param>
    ''' <remarks></remarks>
    Friend Shared Sub iProp_schreiben(ByVal Namen As List(Of String))
        For Each Name As String In Namen 'durchläuft alle in der Liste enthaltenen iProps
            If IV_Doc Is Nothing Then
                MessageBox.Show("Es ist kein Dokument aktiv")
                Exit Sub
            Else
                PropSets = IV_Doc.PropertySets 'Alle Propertysets des akt. Dokumnetes laden
                GetProperty(PropSets, Name).Value = WahlProjekt.iProps.Item(Name)
            End If
        Next
    End Sub

    ''' <summary>
    ''' Fragt eine Liste von iProps ab, ob eine der Props leer ist
    ''' </summary>
    ''' <param name="Namen"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Friend Shared Function iPropDoc_leer(ByVal Namen() As String) As Boolean

        If IV_Doc Is Nothing Then
            MessageBox.Show("Es ist kein Dokument aktiv")
            Exit Function
        Else
            PropSets = IV_Doc.PropertySets 'Alle Propertysets des akt. Dokumnetes laden
            'Durchläuft die Liste der unbedingt benötigten iProps und wertet auf leere iPropps aus
            For n = 0 To Namen.Count - 1
                Dim P_temp As Inventor.Property = GetProperty(PropSets, Namen(n)) 'aktuelle iProp laden
                If P_temp IsNot Nothing Then 'Ist die Eigenschaft vorhanden?
                    If P_temp.Value.ToString.Trim = "" Then 'Die gesuchte iProp ist leer
                        Return True
                    End If
                End If
            Next
        End If
        Return False 'wenn alle iProps nicht leer sind
    End Function

    ''' <summary>
    ''' Gibt eine angefragte Property inkl. Wert zurück. Nothing wenn die Property nicht existiert
    ''' </summary>
    ''' <param name="PropertySets"></param>
    ''' <param name="Suchname"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Friend Shared Function GetProperty(ByVal PropertySets As Inventor.PropertySets, ByVal Suchname As String) As Inventor.Property

        For Each PropSet As Inventor.PropertySet In PropertySets 'Property-Sets durchlaufen
            Debug.Print(PropSet.InternalName & " : " & PropSet.DisplayName)

            For Each Prop As Inventor.Property In PropSet 'iProperties eines Set durchlaufen

                'Abfangen von Property-Werten, die nicht als String darstellbar sind
                Dim Wert As Object
                Try
                    Wert = Prop.Value
                    If Wert Is Nothing Then Wert = ""
                Catch ex As Exception
                    Wert = ""
                End Try

                Debug.Print("Name: " & Prop.Name & " : " & Wert.ToString)
                If Prop.Name = Suchname Then
                    Return Prop
                End If
            Next
        Next
        Return Nothing
    End Function

    ''' <summary>
    ''' Ruft den Dialog für die Erstellung eines neuen Projektes auf und speichert das neue Projekt in den Einstellungen. 
    ''' Es werden max. 8 Projekte gespeichert.
    ''' </summary>
    ''' <remarks></remarks>
    Friend Shared Sub ProjektNeu()
        Using NeuDialog As New Dialg_Neu
            'Dialog aufrufen
            If NeuDialog.ShowDialog = Windows.Forms.DialogResult.OK Then
                'Neues Projekt mit den iProps-Voreinstellungen der Liste hinzufügen
                ProjList.Insert(0, NeuDialog.Projekt_Neu)

                'Liste auf max 8 Positionen kürzen
                Do Until ClassMain.ProjList.Count < 9
                    ProjList.RemoveAt(ClassMain.ProjList.Count - 1)
                Loop

                'Liste in My.Settings speichern
                Projektliste_Save()
            End If
        End Using
    End Sub


#Region "Projektliste in Einstellungen verwalten"
#Region "Projekte aus Settings laden"

    Private Shared Sub Projektliste_Load()
        ProjList = New List(Of Projekt)
        ProjList.Add(Projekt_laden(My.Settings.Pro1))
        ProjList.Add(Projekt_laden(My.Settings.Pro2))
        ProjList.Add(Projekt_laden(My.Settings.Pro3))
        ProjList.Add(Projekt_laden(My.Settings.Pro4))
        ProjList.Add(Projekt_laden(My.Settings.Pro5))
        ProjList.Add(Projekt_laden(My.Settings.Pro6))
        ProjList.Add(Projekt_laden(My.Settings.Pro7))
        ProjList.Add(Projekt_laden(My.Settings.Pro8))

        'Leere Einträge aus Liste entfernen
        For i = ProjList.Count - 1 To 0 Step -1
            If ProjList.Item(i) Is Nothing Then
                ProjList.RemoveAt(i)
            End If
        Next
    End Sub

    Shared Function Projekt_laden(ByVal AL As ArrayList) As Projekt
        If AL Is Nothing OrElse AL.Count < 3 Then
            Return Nothing
        End If
        Projekt_laden = New Projekt(AL.Item(0).ToString, AL.Item(1).ToString, AL.Item(2).ToString)
    End Function
#End Region

#Region "Projekte in Settings speichern"

    Private Shared Sub Projektliste_Save()
        'Liste mit leeren Einträgen (Nothing) auffüllen
        If ProjList.Count < 9 Then
            Dim n As Integer = 8 - ProjList.Count
            Dim _p(n - 1) As Projekt
            ProjList.AddRange(_p)
        End If

        'Settings schreiben und speichern
        My.Settings.Pro1 = Projekt_Save(ProjList.Item(0))
        My.Settings.Pro2 = Projekt_Save(ProjList.Item(1))
        My.Settings.Pro3 = Projekt_Save(ProjList.Item(2))
        My.Settings.Pro4 = Projekt_Save(ProjList.Item(3))
        My.Settings.Pro5 = Projekt_Save(ProjList.Item(4))
        My.Settings.Pro6 = Projekt_Save(ProjList.Item(5))
        My.Settings.Pro7 = Projekt_Save(ProjList.Item(6))
        My.Settings.Pro8 = Projekt_Save(ProjList.Item(7))
        My.Settings.Save()

        'Leere Einträge aus Liste entfernen
        For i = ProjList.Count - 1 To 0 Step -1
            If ProjList.Item(i) Is Nothing Then
                ProjList.RemoveAt(i)
            End If
        Next

    End Sub

    Shared Function Projekt_Save(ByVal P As Projekt) As ArrayList
        If P IsNot Nothing Then
            Dim AL As New ArrayList
            For Each Name As String In NamenListe
                AL.Add(P.iProps.Item(Name))
            Next
            Return AL
        End If
        Return Nothing
    End Function
#End Region
#End Region

End Class
